@@ -1,7 +1,7 @@
 /************ Jdbconn C++ Functions Source Code File (.CPP) ************/
-/*  Name: JDBCONN.CPP  Version 1.0                                     */
+/*  Name: JDBCONN.CPP  Version 1.1                                     */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2016         */
+/*  (C) Copyright to the author Olivier BERTRAND          2016-2017    */
 /*                                                                     */
 /*  This file contains the JDBC connection classes functions.          */
 /***********************************************************************/
@@ -45,9 +45,9 @@
 #include "plgdbsem.h"
 #include "xobject.h"
 #include "xtable.h"
+#include "tabext.h"
 #include "tabjdbc.h"
 //#include "jdbconn.h"
-//#include "plgcnx.h"                       // For DB types
 #include "resource.h"
 #include "valblk.h"
 #include "osutil.h"
@@ -55,9 +55,8 @@
 
 #if defined(__WIN__)
 extern "C" HINSTANCE s_hModule;           // Saved module handle
-#else   // !__WIN__
+#endif   // __WIN__
 #define nullptr 0
-#endif  // !__WIN__
 
 TYPCONV GetTypeConv();
 int GetConvSize();
@@ -121,66 +120,81 @@ int TranslateJDBCType(int stp, char *tn, int prec, int& len, char& v)
 	int type;
 
 	switch (stp) {
-	case -1:  // LONGVARCHAR
-		if (GetTypeConv() != TPC_YES)
-			return TYPE_ERROR;
-		else
-		  len = MY_MIN(abs(len), GetConvSize());
-	case 12:  // VARCHAR
-		v = 'V';
-	case 1:   // CHAR
-		type = TYPE_STRING;
-		break;
-	case 2:   // NUMERIC
-	case 3:   // DECIMAL
-	case -3:  // VARBINARY
-		type = TYPE_DECIM;
-		break;
-	case 4:   // INTEGER
-		type = TYPE_INT;
-		break;
-	case 5:   // SMALLINT
-		type = TYPE_SHORT;
-		break;
-	case -6:  // TINYINT
-	case -7:  // BIT
-		type = TYPE_TINY;
-		break;
-	case 6:   // FLOAT
-	case 7:   // REAL
-	case 8:   // DOUBLE
-		type = TYPE_DOUBLE;
-		break;
-	case 93:  // TIMESTAMP, DATETIME
-		type = TYPE_DATE;
-		len = 19 + ((prec) ? (prec+1) : 0);
-		v = (tn && toupper(tn[0]) == 'T') ? 'S' : 'E';
-		break;
-	case 91:  // DATE, YEAR
-		type = TYPE_DATE;
+		case -1:   // LONGVARCHAR
+		case -16:  // LONGNVARCHAR	(unicode)
+			if (GetTypeConv() != TPC_YES)
+				return TYPE_ERROR;
+			else
+				len = MY_MIN(abs(len), GetConvSize());
+		case 12:   // VARCHAR
+		case -9:   // NVARCHAR	(unicode)
+			v = 'V';
+		case 1:    // CHAR
+		case -15:  // NCHAR	 (unicode)
+		case -8:   // ROWID
+			type = TYPE_STRING;
+			break;
+		case 2:    // NUMERIC
+		case 3:    // DECIMAL
+		case -3:   // VARBINARY
+			type = TYPE_DECIM;
+			break;
+		case 4:    // INTEGER
+			type = TYPE_INT;
+			break;
+		case 5:    // SMALLINT
+			type = TYPE_SHORT;
+			break;
+		case -6:   // TINYINT
+		case -7:   // BIT
+		case 16:   // BOOLEAN
+			type = TYPE_TINY;
+			break;
+		case 6:    // FLOAT
+		case 7:    // REAL
+		case 8:    // DOUBLE
+			type = TYPE_DOUBLE;
+			break;
+		case 93:   // TIMESTAMP, DATETIME
+			type = TYPE_DATE;
+			len = 19 + ((prec) ? (prec + 1) : 0);
+			v = (tn && toupper(tn[0]) == 'T') ? 'S' : 'E';
+			break;
+		case 91:   // DATE, YEAR
+			type = TYPE_DATE;
 
-		if (!tn || toupper(tn[0]) != 'Y') {
-			len = 10;
-			v = 'D';
-		} else {
-			len = 4;
-			v = 'Y';
-		}	// endif len
+			if (!tn || toupper(tn[0]) != 'Y') {
+				len = 10;
+				v = 'D';
+			} else {
+				len = 4;
+				v = 'Y';
+			}	// endif len
 
-		break;
-	case 92:  // TIME
-		type = TYPE_DATE;
-		len = 8 + ((prec) ? (prec+1) : 0);
-		v = 'T';
-		break;
-	case -5:  // BIGINT
-		type = TYPE_BIGINT;
-		break;
-	case 0:   // NULL
-	case -2:  // BINARY
-	case -4:  // LONGVARBINARY
-	default:
-		type = TYPE_ERROR;
+			break;
+		case 92:   // TIME
+			type = TYPE_DATE;
+			len = 8 + ((prec) ? (prec + 1) : 0);
+			v = 'T';
+			break;
+		case -5:   // BIGINT
+			type = TYPE_BIGINT;
+			break;
+		case 0:    // NULL
+		case -2:   // BINARY
+		case -4:   // LONGVARBINARY
+		case 70:   // DATALINK
+		case 2000: // JAVA_OBJECT
+		case 2001: // DISTINCT
+		case 2002: // STRUCT
+		case 2003: // ARRAY
+		case 2004: // BLOB
+		case 2005: // CLOB
+		case 2006: // REF
+		case 2009: // SQLXML
+		case 2011: // NCLOB
+		default:
+			type = TYPE_ERROR;
 		len = 0;
 	} // endswitch type
 
@@ -190,8 +204,8 @@ int TranslateJDBCType(int stp, char *tn, int prec, int& len, char& v)
 /***********************************************************************/
 /*  Allocate the structure used to refer to the result set.            */
 /***********************************************************************/
-static JCATPARM *AllocCatInfo(PGLOBAL g, JCATINFO fid, char *db,
-	char *tab, PQRYRES qrp)
+static JCATPARM *AllocCatInfo(PGLOBAL g, JCATINFO fid, PCSZ db,
+	                            PCSZ tab, PQRYRES qrp)
 {
 	JCATPARM *cap;
 
@@ -214,7 +228,7 @@ static JCATPARM *AllocCatInfo(PGLOBAL g, JCATINFO fid, char *db,
 /*  JDBCColumns: constructs the result blocks containing all columns   */
 /*  of a JDBC table that will be retrieved by GetData commands.        */
 /***********************************************************************/
-PQRYRES JDBCColumns(PGLOBAL g, char *db, char *table, char *colpat,
+PQRYRES JDBCColumns(PGLOBAL g, PCSZ db, PCSZ table, PCSZ colpat,
 	                             int maxres, bool info, PJPARM sjp)
 {
 	int  buftyp[] = {TYPE_STRING, TYPE_STRING, TYPE_STRING, TYPE_STRING,
@@ -317,15 +331,23 @@ PQRYRES JDBCColumns(PGLOBAL g, char *db, char *table, char *colpat,
 /*  JDBCSrcCols: constructs the result blocks containing the              */
 /*  description of all the columns of a Srcdef option.                    */
 /**************************************************************************/
-PQRYRES JDBCSrcCols(PGLOBAL g, char *src, PJPARM sjp)
+PQRYRES JDBCSrcCols(PGLOBAL g, PCSZ src, PJPARM sjp)
 {
+	char    *sqry;
 	PQRYRES  qrp;
 	JDBConn *jcp = new(g)JDBConn(g, NULL);
 
 	if (jcp->Open(sjp))
 		return NULL;
 
-	qrp = jcp->GetMetaData(g, src);
+	if (strstr(src, "%s")) {
+		// Place holder for an eventual where clause
+		sqry = (char*)PlugSubAlloc(g, NULL, strlen(src) + 2);
+		sprintf(sqry, src, "1=1");			 // dummy where clause
+	} else
+		sqry = (char*)src;
+
+	qrp = jcp->GetMetaData(g, sqry);
 	jcp->Close();
 	return qrp;
 } // end of JDBCSrcCols
@@ -334,7 +356,7 @@ PQRYRES JDBCSrcCols(PGLOBAL g, char *src, PJPARM sjp)
 /*  JDBCTables: constructs the result blocks containing all tables in     */
 /*  an JDBC database that will be retrieved by GetData commands.          */
 /**************************************************************************/
-PQRYRES JDBCTables(PGLOBAL g, char *db, char *tabpat, char *tabtyp,
+PQRYRES JDBCTables(PGLOBAL g, PCSZ db, PCSZ tabpat, PCSZ tabtyp,
 	                            int maxres, bool info, PJPARM sjp)
 {
 	int      buftyp[] = {TYPE_STRING, TYPE_STRING, TYPE_STRING,
@@ -525,10 +547,10 @@ JDBConn::JDBConn(PGLOBAL g, TDBJDBC *tdbp)
 		m_Wrap = strcat(strcpy(wn, "wrappers/"), m_Wrap);
 	} // endif m_Wrap
 
-	m_Driver = NULL;
-	m_Url = NULL;
-	m_User = NULL;
-	m_Pwd = NULL;
+//m_Driver = NULL;
+//m_Url = NULL;
+//m_User = NULL;
+//m_Pwd = NULL;
 	m_Ncol = 0;
 	m_Aff = 0;
 	m_Rows = 0;
@@ -772,7 +794,7 @@ bool JDBConn::GetJVM(PGLOBAL g)
 /***********************************************************************/
 int JDBConn::Open(PJPARM sop)
 {
-
+	int      irc = RC_FX;
 	bool		 err = false;
 	jboolean jt = (trace > 0);
 	PGLOBAL& g = m_G;
@@ -819,6 +841,11 @@ int JDBConn::Open(PJPARM sop)
 		jpop->Append(GetPluginDir());
 		jpop->Append("JdbcInterface.jar");
 
+		// All wrappers are pre-compiled in JavaWrappers.jar in the plugin dir
+		jpop->Append(sep);
+		jpop->Append(GetPluginDir());
+		jpop->Append("JavaWrappers.jar");
+
 		//================== prepare loading of Java VM ============================
 		JavaVMInitArgs vm_args;                        // Initialization arguments
 		JavaVMOption* options = new JavaVMOption[N];   // JVM invocation options
@@ -865,30 +892,37 @@ int JDBConn::Open(PJPARM sop)
 		switch (rc) {
 		case JNI_OK:
 			strcpy(g->Message, "VM successfully created");
+			irc = RC_OK;
 			break;
 		case JNI_ERR:
 			strcpy(g->Message, "Initialising JVM failed: unknown error");
-			return RC_FX;
+			break;
 		case JNI_EDETACHED:
 			strcpy(g->Message, "Thread detached from the VM");
-			return RC_FX;
+			break;
 		case JNI_EVERSION:
 			strcpy(g->Message, "JNI version error");
-			return RC_FX;
+			break;
 		case JNI_ENOMEM:
 			strcpy(g->Message, "Not enough memory");
-			return RC_FX;
+			break;
 		case JNI_EEXIST:
 			strcpy(g->Message, "VM already created");
-			return RC_FX;
+			break;
 		case JNI_EINVAL:
 			strcpy(g->Message, "Invalid arguments");
-			return RC_FX;
+			break;
 		default:
-			sprintf(g->Message, "Unknown return code %d", rc);
-			return RC_FX;
+			sprintf(g->Message, "Unknown return code %d", (int)rc);
+			break;
 		} // endswitch rc
 
+		if (trace)
+			htrc("%s\n", g->Message);
+
+		if (irc != RC_OK)
+			return irc;
+
 		//=============== Display JVM version ===============
 		jint ver = env->GetVersion();
 		printf("JVM Version %d.%d\n", ((ver>>16)&0x0f), (ver&0x0f));
@@ -978,10 +1012,10 @@ int JDBConn::Open(PJPARM sop)
 	jobjectArray parms = env->NewObjectArray(4,    // constructs java array of 4
 		env->FindClass("java/lang/String"), NULL);   // Strings
 
-	m_Driver = sop->Driver;
-	m_Url = sop->Url;
-	m_User = sop->User;
-	m_Pwd = sop->Pwd;
+//m_Driver = sop->Driver;
+//m_Url = sop->Url;
+//m_User = sop->User;
+//m_Pwd = sop->Pwd;
 	m_Scrollable = sop->Scrollable;
 	m_RowsetSize = sop->Fsize;
 	//m_LoginTimeout = sop->Cto;
@@ -989,17 +1023,20 @@ int JDBConn::Open(PJPARM sop)
 	//m_UseCnc = sop->UseCnc;
 
 	// change some elements
-	if (m_Driver)
-		env->SetObjectArrayElement(parms, 0, env->NewStringUTF(m_Driver));
+	if (sop->Driver)
+		env->SetObjectArrayElement(parms, 0, env->NewStringUTF(sop->Driver));
 
-	if (m_Url)
-		env->SetObjectArrayElement(parms, 1, env->NewStringUTF(m_Url));
+	if (sop->Url)
+		env->SetObjectArrayElement(parms, 1, env->NewStringUTF(sop->Url));
 
-	if (m_User)
-		env->SetObjectArrayElement(parms, 2, env->NewStringUTF(m_User));
+	if (sop->User)
+		env->SetObjectArrayElement(parms, 2, env->NewStringUTF(sop->User));
 
-	if (m_Pwd)
-		env->SetObjectArrayElement(parms, 3, env->NewStringUTF(m_Pwd));
+	if (sop->Pwd)
+		env->SetObjectArrayElement(parms, 3, env->NewStringUTF(sop->Pwd));
+
+//if (sop->Properties)
+//	env->SetObjectArrayElement(parms, 4, env->NewStringUTF(sop->Properties));
 
 	// call method
 	rc = env->CallIntMethod(job, cid, parms, m_RowsetSize, m_Scrollable);
@@ -1037,7 +1074,7 @@ int JDBConn::Open(PJPARM sop)
 /***********************************************************************/
 /*  Execute an SQL command.                                            */
 /***********************************************************************/
-int JDBConn::ExecSQLcommand(char *sql)
+int JDBConn::ExecSQLcommand(PCSZ sql)
 {
 	int      rc;
 	jint     n;
@@ -1120,7 +1157,7 @@ int JDBConn::Fetch(int pos)
 /***********************************************************************/
 /*  Restart from beginning of result set                               */
 /***********************************************************************/
-int JDBConn::Rewind(char *sql)
+int JDBConn::Rewind(PCSZ sql)
 {
 	int rbuf = -1;
 
@@ -1148,6 +1185,9 @@ void JDBConn::Close()
 		jint      rc;
 		jmethodID did = nullptr;
 
+		// Could have been detached in case of join
+		rc = jvm->AttachCurrentThread((void**)&env, nullptr);
+
 		if (gmID(m_G, did, "JdbcDisconnect", "()I"))
 			printf("%s\n", Msg);
 		else if (Check(env->CallIntMethod(job, did)))
@@ -1175,7 +1215,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 	if (rank == 0)
 		if (!name || (jn = env->NewStringUTF(name)) == nullptr) {
 			sprintf(g->Message, "Fail to allocate jstring %s", SVP(name));
-			longjmp(g->jumper[g->jump_level], TYPE_AM_JDBC);
+			throw (int)TYPE_AM_JDBC;
 		}	// endif name
 
 	// Returns 666 is case of error
@@ -1183,7 +1223,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 
 	if (Check((ctyp == 666) ? -1 : 1)) {
 		sprintf(g->Message, "Getting ctyp: %s", Msg);
-		longjmp(g->jumper[g->jump_level], TYPE_AM_JDBC);
+		throw (int)TYPE_AM_JDBC;
 	} // endif Check
 
 	if (val->GetNullable())
@@ -1200,9 +1240,13 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 
 	switch (ctyp) {
 	case 12:          // VARCHAR
+	case -9:          // NVARCHAR
 	case -1:          // LONGVARCHAR
 	case 1:           // CHAR
-		if (jb)
+	case -15:         // NCHAR
+	case 3:           // DECIMAL
+	case -8:          // ROWID
+		if (jb && ctyp != 3)
 			cn = (jstring)jb;
 		else if (!gmID(g, chrfldid, "StringField", "(ILjava/lang/String;)Ljava/lang/String;"))
 			cn = (jstring)env->CallObjectMethod(job, chrfldid, (jint)rank, jn);
@@ -1219,6 +1263,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 	case 4:           // INTEGER
 	case 5:           // SMALLINT
 	case -6:          // TINYINT
+	case 16:          // BOOLEAN
 	case -7:          // BIT
 		if (!gmID(g, intfldid, "IntField", "(ILjava/lang/String;)I"))
 			val->SetValue((int)env->CallIntMethod(job, intfldid, rank, jn));
@@ -1228,7 +1273,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 		break;
 	case 8:           // DOUBLE
 	case 2:           // NUMERIC
-	case 3:           // DECIMAL
+//case 3:           // DECIMAL
 		if (!gmID(g, dblfldid, "DoubleField", "(ILjava/lang/String;)D"))
 			val->SetValue((double)env->CallDoubleMethod(job, dblfldid, rank, jn));
 		else
@@ -1289,7 +1334,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 			env->DeleteLocalRef(jn);
 
 		sprintf(g->Message, "SetColumnValue: %s rank=%d ctyp=%d", Msg, rank, (int)ctyp);
-		longjmp(g->jumper[g->jump_level], TYPE_AM_JDBC);
+		throw (int)TYPE_AM_JDBC;
 	} // endif Check
 
 	if (rank == 0)
@@ -1300,7 +1345,7 @@ void JDBConn::SetColumnValue(int rank, PSZ name, PVAL val)
 /***********************************************************************/
 /*  Prepare an SQL statement for insert.                               */
 /***********************************************************************/
-bool JDBConn::PrepareSQL(char *sql)
+bool JDBConn::PrepareSQL(PCSZ sql)
 {
 	bool		 b = true;
 	PGLOBAL& g = m_G;
@@ -1323,7 +1368,7 @@ bool JDBConn::PrepareSQL(char *sql)
 /***********************************************************************/
 /*  Execute an SQL query that returns a result set.                    */
 /***********************************************************************/
-int JDBConn::ExecuteQuery(char *sql)
+int JDBConn::ExecuteQuery(PCSZ sql)
 {
 	int			 rc = RC_FX;
 	jint     ncol;
@@ -1351,7 +1396,7 @@ int JDBConn::ExecuteQuery(char *sql)
 /***********************************************************************/
 /*  Execute an SQL query and get the affected rows.                    */
 /***********************************************************************/
-int JDBConn::ExecuteUpdate(char *sql)
+int JDBConn::ExecuteUpdate(PCSZ sql)
 {
 	int      rc = RC_FX;
 	jint     n;
@@ -1379,7 +1424,7 @@ int JDBConn::ExecuteUpdate(char *sql)
 /***********************************************************************/
 /*  Get the number of lines of the result set.                         */
 /***********************************************************************/
-int JDBConn::GetResultSize(char *sql, JDBCCOL *colp)
+int JDBConn::GetResultSize(PCSZ sql, JDBCCOL *colp)
 {
 	int rc, n = 0;
 
@@ -1432,7 +1477,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 	PGLOBAL&   g = m_G;
 	bool       rc = false;
 	PVAL       val = colp->GetValue();
-	jint       n, i = (jint)colp->GetRank();
+	jint       n, jrc = 0, i = (jint)colp->GetRank();
 	jshort     s;
 	jlong      lg;
 //jfloat     f;
@@ -1442,69 +1487,74 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 	jstring    jst = nullptr;
 	jmethodID  dtc, setid = nullptr;
 
-	switch (val->GetType()) {
-	case TYPE_STRING:
-		if (gmID(g, setid, "SetStringParm", "(ILjava/lang/String;)V"))
+	if (val->GetNullable() && val->IsNull()) {
+		if (gmID(g, setid, "SetNullParm", "(II)I"))
 			return true;
 
-		jst = env->NewStringUTF(val->GetCharValue());
-		env->CallVoidMethod(job, setid, i, jst);
-		break;
-	case TYPE_INT:
-		if (gmID(g, setid, "SetIntParm", "(II)V"))
-			return true;
+		jrc = env->CallIntMethod(job, setid, i, (jint)GetJDBCType(val->GetType()));
+	} else switch (val->GetType()) {
+		case TYPE_STRING:
+			if (gmID(g, setid, "SetStringParm", "(ILjava/lang/String;)V"))
+				return true;
 
-		n = (jint)val->GetIntValue();
-		env->CallVoidMethod(job, setid, i, n);
-		break;
-	case TYPE_TINY:
-	case TYPE_SHORT:
-		if (gmID(g, setid, "SetShortParm", "(IS)V"))
-			return true;
+			jst = env->NewStringUTF(val->GetCharValue());
+			env->CallVoidMethod(job, setid, i, jst);
+			break;
+		case TYPE_INT:
+			if (gmID(g, setid, "SetIntParm", "(II)V"))
+				return true;
 
-		s = (jshort)val->GetShortValue();
-		env->CallVoidMethod(job, setid, i, s);
-		break;
-	case TYPE_BIGINT:
-		if (gmID(g, setid, "SetBigintParm", "(IJ)V"))
-			return true;
+			n = (jint)val->GetIntValue();
+			env->CallVoidMethod(job, setid, i, n);
+			break;
+		case TYPE_TINY:
+		case TYPE_SHORT:
+			if (gmID(g, setid, "SetShortParm", "(IS)V"))
+				return true;
 
-		lg = (jlong)val->GetBigintValue();
-		env->CallVoidMethod(job, setid, i, lg);
-		break;
-	case TYPE_DOUBLE:
-	case TYPE_DECIM:
-		if (gmID(g, setid, "SetDoubleParm", "(ID)V"))
-			return true;
+			s = (jshort)val->GetShortValue();
+			env->CallVoidMethod(job, setid, i, s);
+			break;
+		case TYPE_BIGINT:
+			if (gmID(g, setid, "SetBigintParm", "(IJ)V"))
+				return true;
 
-		d = (jdouble)val->GetFloatValue();
-		env->CallVoidMethod(job, setid, i, d);
-		break;
-	case TYPE_DATE:
-		if ((dat = env->FindClass("java/sql/Timestamp")) == nullptr) {
-			strcpy(g->Message, "Cannot find Timestamp class");
-			return true;
-		} else if (!(dtc = env->GetMethodID(dat, "<init>", "(J)V"))) {
-			strcpy(g->Message, "Cannot find Timestamp class constructor");
-			return true;
-		}	// endif's
+			lg = (jlong)val->GetBigintValue();
+			env->CallVoidMethod(job, setid, i, lg);
+			break;
+		case TYPE_DOUBLE:
+		case TYPE_DECIM:
+			if (gmID(g, setid, "SetDoubleParm", "(ID)V"))
+				return true;
 
-		lg = (jlong)val->GetBigintValue() * 1000;
+			d = (jdouble)val->GetFloatValue();
+			env->CallVoidMethod(job, setid, i, d);
+			break;
+		case TYPE_DATE:
+			if ((dat = env->FindClass("java/sql/Timestamp")) == nullptr) {
+				strcpy(g->Message, "Cannot find Timestamp class");
+				return true;
+			} else if (!(dtc = env->GetMethodID(dat, "<init>", "(J)V"))) {
+				strcpy(g->Message, "Cannot find Timestamp class constructor");
+				return true;
+			}	// endif's
 
-		if ((datobj = env->NewObject(dat, dtc, lg)) == nullptr) {
-			strcpy(g->Message, "Cannot make Timestamp object");
-			return true;
-		} else if (gmID(g, setid, "SetTimestampParm", "(ILjava/sql/Timestamp;)V"))
+			lg = (jlong)val->GetBigintValue() * 1000;
+
+			if ((datobj = env->NewObject(dat, dtc, lg)) == nullptr) {
+				strcpy(g->Message, "Cannot make Timestamp object");
+				return true;
+			} else if (gmID(g, setid, "SetTimestampParm", "(ILjava/sql/Timestamp;)V"))
+				return true;
+
+			env->CallVoidMethod(job, setid, i, datobj);
+			break;
+		default:
+			sprintf(g->Message, "Parm type %d not supported", val->GetType());
 			return true;
+		}	// endswitch Type
 
-		env->CallVoidMethod(job, setid, i, datobj);
-		break;
-	default:
-		sprintf(g->Message, "Parm type %d not supported", val->GetType());
-		return true;
-	}	// endswitch Type
-
-	if (Check()) {
+	if (Check(jrc)) {
 		sprintf(g->Message, "SetParam: col=%s msg=%s", colp->GetName(), Msg);
 		rc = true;
 	} // endif msg
@@ -1612,7 +1662,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 	/*  GetMetaData: constructs the result blocks containing the              */
 	/*  description of all the columns of an SQL command.                     */
 	/**************************************************************************/
-	PQRYRES JDBConn::GetMetaData(PGLOBAL g, char *src)
+	PQRYRES JDBConn::GetMetaData(PGLOBAL g, PCSZ src)
 	{
 		static int  buftyp[] = {TYPE_STRING, TYPE_INT, TYPE_INT,
 			                      TYPE_INT,    TYPE_INT};
@@ -1814,7 +1864,7 @@ bool JDBConn::SetParam(JDBCCOL *colp)
 		PGLOBAL& g = m_G;
 //	void    *buffer;
 		int      i, ncol;
-		PSZ      fnc = "Unknown";
+		PCSZ     fnc = "Unknown";
 		uint     n;
 		short    len, tp;
 		int      crow = 0;
