@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2000, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2013, 2016, MariaDB Corporation.
+Copyright (c) 2013, 2017, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -167,6 +167,8 @@ class ha_innobase: public handler
 	int index_first(uchar * buf);
 	int index_last(uchar * buf);
 
+	bool has_gap_locks() const { return true; }
+
 	int rnd_init(bool scan);
 	int rnd_end();
 	int rnd_next(uchar *buf);
@@ -318,14 +320,6 @@ class ha_innobase: public handler
 
 	bool check_if_supported_virtual_columns(void) { return TRUE; }
 
-	/** This function reads zip dict-related info from SYS_ZIP_DICT
-	and SYS_ZIP_DICT_COLS for all columns marked with
-	COLUMN_FORMAT_TYPE_COMPRESSED flag and updates
-	zip_dict_name / zip_dict_data for those which have associated
-	compression dictionaries.
-	*/
-	virtual void update_field_defs_with_zip_dict_info();
-
 private:
 	/** Builds a 'template' to the prebuilt struct.
 
@@ -433,6 +427,15 @@ int thd_slave_thread(const MYSQL_THD thd);
 */
 int thd_non_transactional_update(const MYSQL_THD thd);
 
+/**
+  Get high resolution timestamp for the current query start time.
+  The timestamp is not anchored to any specific point in time,
+  but can be used for comparison.
+
+  @retval timestamp in microseconds precision
+*/
+unsigned long long thd_start_utime(const MYSQL_THD thd);
+
 /**
   Get the user thread's binary logging format
   @param thd  user thread
@@ -537,7 +540,7 @@ innobase_index_name_is_reserved(
 	const KEY*	key_info,	/*!< in: Indexes to be created */
 	ulint		num_of_keys)	/*!< in: Number of indexes to
 					be created. */
-	MY_ATTRIBUTE((nonnull, warn_unused_result));
+	MY_ATTRIBUTE((nonnull(1), warn_unused_result));
 
 /*****************************************************************//**
 #ifdef WITH_WSREP
@@ -741,31 +744,3 @@ ib_push_frm_error(
 	TABLE*		table,		/*!< in: MySQL table */
 	ulint		n_keys,		/*!< in: InnoDB #keys */
 	bool		push_warning);	/*!< in: print warning ? */
-
-/** This function checks if all the compression dictionaries referenced
-in table->fields exist in SYS_ZIP_DICT InnoDB system table.
-@return true if all referenced dictionaries exist */
-UNIV_INTERN
-bool
-innobase_check_zip_dicts(
-	const TABLE*	table,		/*!< in: table in MySQL data
-					dictionary */
-	ulint*		dict_ids,	/*!< out: identified zip dict ids
-					(at least n_fields long) */
-	trx_t*		trx,		/*!< in: transaction */
-	const char**	err_dict_name);	/*!< out: the name of the
-					zip_dict which does not exist. */
-
-/** This function creates compression dictionary references in
-SYS_ZIP_DICT_COLS InnoDB system table for table_id based on info
-in table->fields and provided zip dict ids. */
-UNIV_INTERN
-void
-innobase_create_zip_dict_references(
-	const TABLE*	table,		/*!< in: table in MySQL data
-					dictionary */
-	table_id_t	ib_table_id,	/*!< in: table ID in Innodb data
-					dictionary */
-	ulint*		zip_dict_ids,	/*!< in: zip dict ids
-					(at least n_fields long) */
-	trx_t*		trx);		/*!< in: transaction */
